package News::Article;
use strict;

use GnuPG::Interface;
use IO::Handle;

sub sign_control {
	my ($self, $keyid, $passphrase, @extra) = @_;

	my @headers = qw(subject control message-id date from sender);
	push @headers, map { lc $_ } @extra;

	# Check to make sure we have the required headers.
	foreach (qw(subject control from)) {
		return "Required header $_ missing" unless $self->{Headers}{$_};
	}
	$self->add_message_id('cmsg-') unless $self->{Headers}{'message-id'};
	$self->add_date();

	# We have to sign the list of headers and each header on a seperate
	# line. Note that the verification code doesn't support continuation
	# headers, so be careful not to use them when calling this method.
	my $signheads = join(',', map { canonical($_) } @headers);
	my @sign;
	push @sign, 'X-Signed-Headers: ' . $signheads;
	foreach (@headers) {
		push @sign, canonical($_).": ".($self->header($_) || '');
	}

	my ($input, $output, $error)
		= (IO::Handle->new, IO::Handle->new, IO::Handle->new);
	my $handles = GnuPG::Handles->new(stdin => $input, stdout => $output,
		stderr => $error);

	local $SIG{PIPE} = 'IGNORE';
	my $gnupg = GnuPG::Interface->new;
	$gnupg->options->meta_interactive(0);
#	$gnupg->options->homedir(...);
	$gnupg->options->default_key($keyid) if $keyid;
	$gnupg->passphrase($passphrase) if $passphrase;
	$gnupg->clearsign(handles => $handles);

	# Now send everything to PGP to sign. We have to add a new line to the
	# end of every line of the body, since we're storing it without them.
	# Make sure we munge for attached signatures, since pgpverify tests with
	# an attached signature.
	# An extra \n must be appended, I don't understand why.
	my $pid = fork;
	if (not $pid) {
		push @sign, '', @{$self->{Body}};
		print $input join("\n", map { s/\s+$//; $_ } @sign) . "\n\n";
		close $input;
		exit 0;
	}
	close $input;

	my @signature = <$output>;
	close $output;
	my @err = <$error>;
	close $error;
	die "BUG (ret=$?)" if waitpid($pid, 0) < 0 or $? > 0;
	die join('', @err) if wait < 0 or $? > 0;

	while (local $_ = shift @signature) {
		last if /^-----BEGIN PGP SIGNATURE-----$/;
		die 'No signature from PGP' if not @signature;
	}

	my $version;
	while (local $_ = shift @signature) {
		last if /^$/;
		$version = $1 if /^Version: GnuPG v(\S+)/;
	}
	$version ||= 'unknown';

	my $signature;
	while (local $_ = shift @signature) {
		last if /^-----END PGP SIGNATURE-----$/;
		chop;
		$signature .= "\n\t$_";
	}

	$self->add_headers('x-pgp-sig', "$version $signheads$signature");
	return ();
}

1;
